unit oTermMappingTable;

interface

uses
  Classes,
  SysUtils,
  iCPTerminology,
  XMLIntf,
  XMLDoc,
  uXML;

type
  TTermMappingTable = class(TInterfacedObject, ITermMappingTable)
  private
    fChangeMade: boolean;
    fFileName: string;
    fXMLDoc: IXMLDocument;
    fTerminology: ITerminologyController;
    fEntryNode: IXMLNode;
    fHeaderNode: IXMLNode;
    fOnStatusUpdate: TOnStatusUpdate;

    fCreatedBy: string;
    fCreatedDateTime: TDateTime;
    fCreatedOnWkstn: string;
    fUpdatedBy: string;
    fUpdatedDateTime: TDateTime;
    fUpdatedOnWkstn: string;
    fID: string;
    fName: string;
    fActive: boolean;
    fComment: string;
    fSourceID: string;
    fSourceApplication: string;
    fSourceVersion: string;
    fSourceTrusted: boolean;
    fHL7SendingApplication: string;

    fEntries: IInterfaceList;

    function getCreatedBy: string; stdcall;
    function getCreatedDateTime: TDateTime; stdcall;
    function getCreatedOnWkstn: string; stdcall;
    function getUpdatedBy: string; stdcall;
    function getUpdatedDateTime: TDateTime; stdcall;
    function getUpdatedOnWkstn: string; stdcall;
    function getID: string; stdcall;
    function getName: string; stdcall;
    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getSourceID: string; stdcall;
    function getSourceApplication: string; stdcall;
    function getSourceVersion: string; stdcall;
    function getSourceTrusted: boolean; stdcall;
    function getHL7SendingApplication: string; stdcall;
    function getIsOpen: boolean; stdcall;
    function getFileName: string; stdcall;
    function getChangeMade: boolean; stdcall;

    procedure setUpdatedBy(const aValue: string); stdcall;
    procedure setUpdatedDateTime(const aValue: TDateTime); stdcall;
    procedure setUpdatedOnWkstn(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;
    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setSourceID(const aValue: string); stdcall;
    procedure setSourceApplication(const aValue: string); stdcall;
    procedure setSourceVersion(const aValue: string); stdcall;
    procedure setSourceTrusted(const aValue: boolean); stdcall;
    procedure setHL7SendingApplication(const aValue: string); stdcall;
    procedure setOnStatusUpdate(aProcedure: TOnStatusUpdate); stdcall;
    procedure setFileName(const aValue: string); stdcall;
    procedure setChangeMade(const aValue: boolean); stdcall;

    function NewEntry(aXMLNode: IXMLNode): ITermMappingTableEntry; overload;
    function NewEntry(aKey: string): ITermMappingTableEntry; overload;

    procedure DeleteEntry(aTermMappingTableEntry: ITermMappingTableEntry);
    function NewEnumerator: ITermMappingTableEnumerator;

    procedure Close; stdcall;
    procedure Load(aFileName: string); stdcall;
    procedure Save; stdcall;
  public
    constructor Create(aTerminology: ITerminologyController);
    destructor Destroy; override;
  end;

implementation

uses
  oTermMappingTableEntry,
  oTermMappingTableEnumerator;

const
  XML_CREATED_DATE_TIME = 'CREATED_DATE_TIME';
  XML_CREATED_BY = 'CREATED_BY';
  XML_CREATED_ON_WKSTN = 'CREATED_ON_WKSTN';
  XML_UPDATED_DATE_TIME = 'UPDATED_DATE_TIME';
  XML_UPDATED_BY = 'UPDATED_BY';
  XML_UPDATED_ON_WKSTN = 'UPDATED_ON_WKSTN';
  XML_ID = 'ID';
  XML_NAME = 'NAME';
  XML_ACTIVE = 'ACTIVE';
  XML_COMMENT = 'COMMENT';
  XML_SOURCE_ID = 'SOURCE_ID';
  XML_SOURCE_APPLICATION = 'SOURCE_APPLICATION';
  XML_SOURCE_VERSION = 'SOURCE_VERSION';
  XML_SOURCE_TRUSTED = 'SOURCE_TRUSTED';
  XML_HL7_SENDING_APPLICATION = 'HL7_SENDING_APPLICATION';
  XML_HEADER = 'HEADER';
  XML_ENTRIES = 'ENTRIES';

constructor TTermMappingTable.Create(aTerminology: ITerminologyController);
begin
  inherited Create;
  fXMLDoc := NewXMLDocument;
  fTerminology := aTerminology;
  TInterfaceList.Create.GetInterface(IInterfaceList, fEntries);
end;

destructor TTermMappingTable.Destroy;
begin
  Close;
  fTerminology := nil;
  fEntries := nil;
  inherited;
end;

procedure TTermMappingTable.DeleteEntry(aTermMappingTableEntry: ITermMappingTableEntry);
begin
  try
    fEntryNode.ChildNodes.Delete(fEntryNode.ChildNodes.IndexOf(aTermMappingTableEntry.getXMLNode));
    fEntries.Delete(fEntries.IndexOf(aTermMappingTableEntry));
  except
    raise
      ETerminologyDictionaryError.Create('Error deleting mapping table entry.');
  end;
  fChangeMade := True;
end;

procedure TTermMappingTable.Close;
begin
  fFileName := '';
  fXMLDoc := nil;
  fEntries.Clear;
end;

procedure TTermMappingTable.Load(aFileName: string);
var
  i: integer;
  aNode: IXMLNode;
  aEntry: ITermMappingTableEntry;
begin
  try
    fEntries.Clear;
    fXMLDoc := NewXMLDocument;
    fFileName := aFileName;

    if not FileExists(fFileName) then
      try
        fXMLDoc.DocumentElement := fXMLDoc.AddChild('MAPPING_TABLE');
        with fXMLDoc.DocumentElement.AddChild(XML_HEADER) do
          begin
            ChildValues[XML_CREATED_DATE_TIME] := DateTimeToXMLString(Now);
            ChildValues[XML_CREATED_BY] := GetUserFromWindows;
            ChildValues[XML_CREATED_ON_WKSTN] := GetComputerNetName;
            ChildValues[XML_UPDATED_DATE_TIME] := DateTimeToXMLString(Now);
            ChildValues[XML_UPDATED_BY] := GetUserFromWindows;
            ChildValues[XML_UPDATED_ON_WKSTN] := GetComputerNetName;
            ChildValues[XML_ID] := NewGUID;
            ChildValues[XML_NAME] := 'NEW MAPPING TABLE';
            ChildValues[XML_ACTIVE] := BOOLEAN_TO_INT[False];
            ChildValues[XML_COMMENT] := 'New Vendor Mapping Table';
          end;
        fXMLDoc.DocumentElement.AddChild(XML_ENTRIES);
        fXMLDoc.SaveToFile(fFileName);
      except
        raise Exception.Create('Unable to create new terminology file.');
      end
    else
      try
        fXMLDoc.LoadFromFile(aFileName);
      except
        raise Exception.Create('Error loading terminology file.');
      end;

    fXMLDoc.Active := True;
    fEntryNode := fXMLDoc.DocumentElement.ChildNodes.FindNode(XML_ENTRIES);
    fHeaderNode := fXMLDoc.DocumentElement.ChildNodes.FindNode(XML_HEADER);

    fCreatedBy := XMLValueAsStr(fHeaderNode, XML_CREATED_BY);
    fCreatedDateTime := XMLValueAsDateTime(fHeaderNode, XML_CREATED_DATE_TIME);
    fCreatedOnWkstn := XMLValueAsStr(fHeaderNode, XML_CREATED_ON_WKSTN);

    fUpdatedBy := XMLValueAsStr(fHeaderNode, XML_UPDATED_BY);
    fUpdatedDateTime := XMLValueAsDateTime(fHeaderNode, XML_CREATED_DATE_TIME);
    fUpdatedOnWkstn := XMLValueAsStr(fHeaderNode, XML_UPDATED_ON_WKSTN);

    fID := XMLValueAsStr(fHeaderNode, XML_ID);
    fName := XMLValueAsStr(fHeaderNode, XML_NAME);
    fActive := (XMLValueAsInt(fHeaderNode, XML_ACTIVE, 0) = 1);
    fComment := XMLValueAsStr(fHeaderNode, XML_COMMENT);
    fSourceID := XMLValueAsStr(fHeaderNode, XML_SOURCE_ID);
    fSourceApplication := XMLValueAsStr(fHeaderNode, XML_SOURCE_APPLICATION);
    fSourceVersion := XMLValueAsStr(fHeaderNode, XML_SOURCE_VERSION);
    fSourceTrusted := (XMLValueAsInt(fHeaderNode, XML_SOURCE_TRUSTED, 0) = 1);
    fHL7SendingApplication := XMLValueAsStr(fHeaderNode, XML_HL7_SENDING_APPLICATION);

    aNode := fEntryNode.ChildNodes.First;
    i := 0;
    while Assigned(aNode) do
      try
        inc(i);
        if Assigned(fOnStatusUpdate) then
          fOnStatusUpdate(Format('Loading Mapping Table - %s Complete...', [getPercentageDone(i, fEntryNode.ChildNodes.Count)]));
        aEntry := NewEntry(aNode);
        aEntry.Load;
      finally
        aNode := aNode.NextSibling;
      end;

    if Assigned(fOnStatusUpdate) then
      fOnStatusUpdate;
  except
    on E: Exception do
      raise Exception.CreateFmt('Error loading file %s. Error: %s', [aFileName, E.Message]);
  end;
end;

function TTermMappingTable.NewEntry(aXMLNode: IXMLNode): ITermMappingTableEntry;
begin
  if TTermMappingTableEntry.Create((Self as ITermMappingTable),fTerminology, aXMLNode).GetInterface(ITermMappingTableEntry, Result) then
    fEntries.Add(Result)
  else
    Result := nil;
end;

function TTermMappingTable.NewEntry(aKey: string): ITermMappingTableEntry;
begin
  if NewEntry(fEntryNode.AddChild('ENTRY')).QueryInterface(ITermMappingTableEntry, Result) = 0 then
    begin
      Result.setVendorKey(aKey);
      Result.setDoNotProcess(True);
      fChangeMade := True;
    end
  else
    Result := nil;
end;

function TTermMappingTable.NewEnumerator: ITermMappingTableEnumerator;
begin
  TTermMappingTableEnumerator.Create(fEntries).GetInterface(ITermMappingTableEnumerator, Result);
end;

procedure TTermMappingTable.Save;
begin
  fHeaderNode.ChildNodes.Clear;
  fHeaderNode.ChildValues[XML_CREATED_BY] := fCreatedBy;
  fHeaderNode.ChildValues[XML_CREATED_DATE_TIME] := DateTimeToXMLString(fCreatedDateTime);
  fHeaderNode.ChildValues[XML_CREATED_ON_WKSTN] := fCreatedOnWkstn;
  fHeaderNode.ChildValues[XML_UPDATED_BY] := fUpdatedBy;
  fHeaderNode.ChildValues[XML_UPDATED_DATE_TIME] := DateTimeToXMLString(fUpdatedDateTime);
  fHeaderNode.ChildValues[XML_CREATED_ON_WKSTN] := fUpdatedOnWkstn;
  fHeaderNode.ChildValues[XML_ID] := fID;
  fHeaderNode.ChildValues[XML_NAME] := fName;
  fHeaderNode.ChildValues[XML_ACTIVE] := BOOLEAN_TO_INT[fActive];
  fHeaderNode.ChildValues[XML_COMMENT] := fComment;
  fHeaderNode.ChildValues[XML_SOURCE_ID] := fSourceID;
  fHeaderNode.ChildValues[XML_SOURCE_APPLICATION] := fSourceApplication;
  fHeaderNode.ChildValues[XML_SOURCE_VERSION] := fSourceVersion;
  fHeaderNode.ChildValues[XML_SOURCE_TRUSTED] := BOOLEAN_TO_INT[fSourceTrusted];
  fHeaderNode.ChildValues[XML_HL7_SENDING_APPLICATION] := fHL7SendingApplication;

  with NewEnumerator do
    while MoveNext do
      GetCurrentEntry.Save;

  fXMLDoc.SaveToFile(fFileName);
  fChangeMade := False;
end;

{ private getters and setters }

function TTermMappingTable.getActive: boolean;
begin
  Result := fActive;
end;

function TTermMappingTable.getChangeMade: boolean;
begin
  Result := fChangeMade;
end;

function TTermMappingTable.getComment: string;
begin
  Result := fComment;
end;

function TTermMappingTable.getCreatedBy: string;
begin
  Result := fCreatedBy;
end;

function TTermMappingTable.getCreatedDateTime: TDateTime;
begin
  Result := fCreatedDateTime;
end;

function TTermMappingTable.getCreatedOnWkstn: string;
begin
  Result := fCreatedOnWkstn;
end;

function TTermMappingTable.getFileName: string;
begin
  Result := fFileName;
end;

function TTermMappingTable.getHL7SendingApplication: string;
begin
  Result := fHL7SendingApplication;
end;

function TTermMappingTable.getID: string;
begin
  Result := fID;
end;

function TTermMappingTable.getIsOpen: boolean;
begin
  Result := (fFileName <> '');
end;

function TTermMappingTable.getName: string;
begin
  Result := fName;
end;

function TTermMappingTable.getSourceApplication: string;
begin
  Result := fSourceApplication;
end;

function TTermMappingTable.getSourceID: string;
begin
  Result := fSourceID;
end;

function TTermMappingTable.getSourceTrusted: boolean;
begin
  Result := fSourceTrusted;
end;

function TTermMappingTable.getSourceVersion: string;
begin
  Result := fSourceVersion;
end;

function TTermMappingTable.getUpdatedBy: string;
begin
  Result := fUpdatedBy;
end;

function TTermMappingTable.getUpdatedDateTime: TDateTime;
begin
  Result := fUpdatedDateTime;
end;

function TTermMappingTable.getUpdatedOnWkstn: string;
begin
  Result := fUpdatedOnWkstn
end;

procedure TTermMappingTable.setActive(const aValue: boolean);
begin
  fActive := aValue;
  fChangeMade := True;
end;

procedure TTermMappingTable.setChangeMade(const aValue: boolean);
begin
  {Allows individual entries to do the same}
  fChangeMade := aValue;
end;

procedure TTermMappingTable.setComment(const aValue: string);
begin
  fComment := aValue;
  fChangeMade := True;
end;

procedure TTermMappingTable.setFileName(const aValue: string);
begin
  fFileName := aValue;
end;

procedure TTermMappingTable.setHL7SendingApplication(const aValue: string);
begin
  fHL7SendingApplication := aValue;
  fChangeMade := True;
end;

procedure TTermMappingTable.setName(const aValue: string);
begin
  fName := aValue;
  fChangeMade := True;
end;

procedure TTermMappingTable.setOnStatusUpdate(aProcedure: TOnStatusUpdate);
begin
  fOnStatusUpdate := aProcedure;
end;

procedure TTermMappingTable.setSourceApplication(const aValue: string);
begin
  fSourceApplication := aValue;
  fChangeMade := True;
end;

procedure TTermMappingTable.setSourceID(const aValue: string);
begin
  fSourceID := aValue;
  fChangeMade := True;
end;

procedure TTermMappingTable.setSourceTrusted(const aValue: boolean);
begin
  fSourceTrusted := aValue;
  fChangeMade := True;
end;

procedure TTermMappingTable.setSourceVersion(const aValue: string);
begin
  fSourceVersion := aValue;
  fChangeMade := True;
end;

procedure TTermMappingTable.setUpdatedBy(const aValue: string);
begin
  fUpdatedBy := aValue;
  fChangeMade := True;
end;

procedure TTermMappingTable.setUpdatedDateTime(const aValue: TDateTime);
begin
  fUpdatedDateTime := aValue;
  fChangeMade := True;
end;

procedure TTermMappingTable.setUpdatedOnWkstn(const aValue: string);
begin
  fUpdatedOnWkstn := aValue;
  fChangeMade := True;
end;

end.

