unit oTermUnitConversion;

interface

uses
  uXML,
  XMLIntf,
  SysUtils,
  ComCtrls,
  Classes,
  oTermComponent,
  iCPTerminology;

type
  TTermUnitConversion = class(TTermComponent, ITermComponent, ITermUnitConversion)
  private
    fConversionFactor: double;
    fConversionOffsetPre: double;
    fConversionOffsetPost: double;
    fDecimalPrecision: integer;
    fUnitsAreEqual: boolean;
  public
    constructor Create(aParentTerm: ITerm; aUnit: ITerm);
    destructor Destroy; override;

    function getConversionFactor: double; stdcall;
    function getConversionOffsetPost: double; stdcall;
    function getConversionOffsetPre: double; stdcall;
    function getDecimalPrecision: integer; stdcall;
    function getUnitsAreEqual: boolean; stdcall;
    function getComponentDisplayName: string; override; stdcall;

    procedure setConversionFactor(const aValue: double); stdcall;
    procedure setConversionOffsetPost(const aValue: double); stdcall;
    procedure setConversionOffsetPre(const aValue: double); stdcall;
    procedure setDecimalPrecision(const aValue: integer); stdcall;
    procedure setUnitsAreEqual(const aValue: boolean); stdcall;

    procedure Save(aXMLNode: IXMLNode); override; stdcall;
    procedure Load(aXMLNode: IXMLNode); override; stdcall;
    procedure Validate; override; stdcall;
  end;

implementation

const
  {Default XML Tags for the properties}
  XML_CONVERSION_OFFSET_PRE = 'CONVERSION_OFFSET_PRE';
  XML_CONVERSION_OFFSET_POST = 'CONVERSION_OFFSET_POST';
  XML_CONVERSION_FACTOR = 'CONVERSION_FACTOR';
  XML_DECIMAL_PRECISION = 'DECIMAL_PRECISION';
  XML_UNITS_ARE_EQUAL = 'UNITS_ARE_EQUAL';

  {Default values for the properties}
  DEFAULT_CONVERSION_OFFSET_PRE = 0.0;
  DEFAULT_CONVERSION_OFFSET_POST = 0.0;
  DEFAULT_CONVERSION_FACTOR = 1.0;
  DEFAULT_DECIMAL_PRECISION = 2;
  DEFAULT_UNITS_ARE_EQUAL = 0;

constructor TTermUnitConversion.Create(aParentTerm: ITerm; aUnit: ITerm);
begin
  inherited Create(ctConversion, aParentTerm, aUnit);
  fConversionFactor := DEFAULT_CONVERSION_FACTOR;
  fConversionOffsetPre := DEFAULT_CONVERSION_OFFSET_PRE;
  fConversionOffsetPost := DEFAULT_CONVERSION_OFFSET_POST;
  fDecimalPrecision := DEFAULT_DECIMAL_PRECISION;
end;

destructor TTermUnitConversion.Destroy;
begin
  inherited;
end;

procedure TTermUnitConversion.Load(aXMLNode: IXMLNode);
begin
  try
    inherited Load(aXMLNode);
    fConversionOffsetPre := XMLValueAsDouble(aXMLNode, XML_CONVERSION_OFFSET_PRE, DEFAULT_CONVERSION_OFFSET_PRE);
    fConversionOffsetPost := XMLValueAsDouble(aXMLNode, XML_CONVERSION_OFFSET_POST, DEFAULT_CONVERSION_OFFSET_POST);
    fConversionFactor := XMLValueAsDouble(aXMLNode, XML_CONVERSION_FACTOR, DEFAULT_CONVERSION_FACTOR);
    fDecimalPrecision := XMLValueAsInt(aXMLNode, XML_DECIMAL_PRECISION, DEFAULT_DECIMAL_PRECISION);
    fUnitsAreEqual := XMLValueAsInt(aXMLNode, XML_UNITS_ARE_EQUAL, DEFAULT_UNITS_ARE_EQUAL) = 1;
  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to load %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

procedure TTermUnitConversion.Save(aXMLNode: IXMLNode);
begin
  try
    inherited Save(aXMLNode);
    aXMLNode.ChildValues[XML_CONVERSION_OFFSET_PRE] := fConversionOffsetPre;
    aXMLNode.ChildValues[XML_CONVERSION_OFFSET_POST] := fConversionOffsetPost;
    aXMLNode.ChildValues[XML_CONVERSION_FACTOR] := fConversionFactor;
    aXMLNode.ChildValues[XML_DECIMAL_PRECISION] := fDecimalPrecision;
    aXMLNode.ChildValues[XML_UNITS_ARE_EQUAL] := BOOLEAN_TO_INT[fUnitsAreEqual];
  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to load %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

{ Private Getters }

function TTermUnitConversion.getComponentDisplayName: string;
begin
  Result := getComponentTerm.getTerm;
end;

function TTermUnitConversion.getConversionFactor: double;
begin
  Result := fConversionFactor;
end;

function TTermUnitConversion.getConversionOffsetPost: double;
begin
  Result := fConversionOffsetPost;
end;

function TTermUnitConversion.getConversionOffsetPre: double;
begin
  Result := fConversionOffsetPre;
end;

function TTermUnitConversion.getDecimalPrecision: integer;
begin
  Result := fDecimalPrecision;
end;

function TTermUnitConversion.getUnitsAreEqual: boolean;
begin
  Result := fUnitsAreEqual;
end;

{ Private Setters }

procedure TTermUnitConversion.setConversionFactor(const aValue: double);
begin
  fConversionFactor := aValue;
end;

procedure TTermUnitConversion.setConversionOffsetPost(const aValue: double);
begin
  fConversionOffsetPost := aValue;
end;

procedure TTermUnitConversion.setConversionOffsetPre(const aValue: double);
begin
  fConversionOffsetPre := aValue;
end;

procedure TTermUnitConversion.setDecimalPrecision(const aValue: integer);
begin
  fDecimalPrecision := aValue;
end;

procedure TTermUnitConversion.setUnitsAreEqual(const aValue: boolean);
begin
  fUnitsAreEqual := aValue;
end;

procedure TTermUnitConversion.Validate;
begin
  inherited;
  if getComponentTerm = nil then
    getParentTerm.NewError(etComponentSetting, 'No post unit assigned to this unit conversion')
  else
    if getComponentTerm.getTermType <> ttUnit then
      getParentTerm.NewError(etComponentSetting, getComponentTerm.getTerm + ' non unit term assigned to this unit conversion')
    else
      if (getParentTerm.getID = getComponentTerm.getID) then
        getParentTerm.NewError(etComponentSetting, getComponentTerm.getTerm + ' unnecessary conversion record -- same terms')
      else
        begin
          if (fConversionFactor = 1) and (fConversionOffsetPre = 0) and (fConversionOffsetPost = 0) and (not fUnitsAreEqual) then
            getParentTerm.NewError(etComponentSetting, getComponentTerm.getTerm +
              ' possible invalid conversion record no conversion will occur (x=(y+0)*1+0)');

          if (fDecimalPrecision < 0) or (fDecimalPrecision > 8) then
            getParentTerm.NewError(etComponentSetting, getComponentTerm.getTerm + ' invalid value for decimal precision (0-8 only)');
        end;
end;

end.

